import Catalog from "../common/Catalog";
import { useEffect, useState } from "react";

import DataService from "../../services/DataService"

let headers = [
  {label: "name", text: "Имя"},
  {label: "genre", text: "Жанр"},
  {label: "fullNames", text: "Актеры"}
]


export default function CatalogFilms() {
  const url = "/films";

  const [data, setData] = useState({
    name: "",
    genre: [],
    fullNames: []
  });

  const [selectedData, setSelectedData] = useState({
    genre: "",
    actor: ""
  });

  const [genres, setGenres] = useState([]);
  const [actors, setActors] = useState([]);

  useEffect(() => {
    let genre;
    DataService.readAll("/genres").then(res => {
      setGenres(res)
      genre = res[0].name;
    }).then(() => (
      DataService.readAll("/actors")
    )).then(res => {
      setActors(res)
      setSelectedData({genre, actor: res[0].name + " " + res[0].surname});
    });
  }, [])


  function handleFormChange(e) {
    console.log(e.target.value);
    setData({ ...data, [e.target.id]: e.target.value })
  }

  function handleChangeSelected(e) {
    setSelectedData({...selectedData, [e.target.id]: e.target.value});
  }

  function addGenre() {
    for(let g of data.genre) {
      if(g == selectedData.genre) return;
    }
    let temp = data.genre.slice(0);
    temp.push(selectedData.genre);
    setData({...data, genre: temp});
  }

  function addActor() {
    for(let a of data.fullNames) {
      if(a == selectedData.actor) return;
    }
    let temp = data.fullNames.slice(0);
    temp.push(selectedData.actor);
    setData({...data, fullNames: temp});
  }

  function deleteGenre(e) {
    let genreName = e.target.previousSibling.textContent;
    let temp = data.genre.filter(x => (x != genreName));
    setData({...data, genre: temp})
  }

  function deleteActor(e) {
    let actorName = e.target.previousSibling.textContent;
    let temp = data.fullNames.filter(x => (x != actorName));
    setData({...data, fullNames: temp})
  }

  function onAdd() {
    setData({name: "", genre: [], fullNames: []});
  }

  
  function onChange(data) {
    setData(data);
  }

  function validate(data) {
    if(data.name === "") return false;
    return true;
  }

  return (
    <Catalog url={url} headers={headers} data={data} onAdd={onAdd} onChange={onChange} validate={validate}>
      <div>
        <div className="mb-3">
            <label htmlFor="name" className="form-label">Название</label>
            <input type="text" value={data.name} id="name" className="form-control" required autoComplete="off"
                onChange={handleFormChange}/>
        </div>
        <select id="genre" className="form-select" required
                value={selectedData.genre} onChange={handleChangeSelected}>
                <option disabled value="">Укажите жанр</option>
                { 
                    genres.map(({name, id}) => 
                        <option key={id} value={name}>{name}</option>
                    )
                }
        </select>
        {
          data.genre.map(value => (
              <div className="badge bg-secondary m-1" key={value}>
                <span>{value}</span>
                <button className="btn-close bg-danger m-1" onClick={deleteGenre}></button>
              </div>
          ))
        }
        <br></br>
        <button onClick={addGenre} className="btn btn-success">Добавить жанр</button>

        <select id="actor" className="form-select" required
                value={selectedData.actor} onChange={handleChangeSelected}>
                <option disabled value="">Укажите актера</option>
                { 
                    actors.map(({name, surname, id}) => 
                        <option key={id} value={name + " " + surname}>{name + " " + surname}</option>
                    )
                }
        </select>
        {
          data.fullNames.map(value => (
              <div className="badge bg-secondary m-1" key={value}>
                <span>{value}</span>
                <button className="btn-close bg-danger m-1" onClick={deleteActor}></button>
              </div>
          ))
        }
        <br></br>
        <button onClick={addActor} className="btn btn-success">Добавить актера</button>
        </div>
    </Catalog>
  )
}