import { useEffect, useState } from "react";
import Table from "./Table";
import Toolbar from "./Toolbar";
import Modal from "./Modal";
import DataService from "../../services/DataService"

export default function Catalog(props) {
  const [items, setItems] = useState([]);
  const [visible, setVisible] = useState(false);
  const [modalConfirm, setModalConfirm] = useState("");
  const [modalHeader, setModalHeader] = useState("");
  const [isEdit, setEdit] = useState(false);
  let selectedItems = new Set();

  useEffect(() => {
    loadItems();
  }, [])

  function toggleSelect(id) {
    if(selectedItems.has(id)) selectedItems.delete(id);
    else selectedItems.add(id);
  }

  function loadItems() {
    DataService.readAll(props.url)
      .then(res => setItems(res));
  }

  function saveData() {
    if(!props.validate(props.data)) return;
    if(!isEdit) {
      DataService.create(props.url, props.data).then(() => loadItems());
    } else {
      console.log(props.data);  
      DataService.update(props.url, props.data, props.data.id).then(() => loadItems());
    }
  }

  function addHandler() {
    setModalHeader("Добавление");
    setModalConfirm("Добавить");
    setEdit(false);
    setVisible(true);
    props.onAdd();
  }

  function changeHandler() {
    if(!selectedItems.size) return;
    if(!items.length) return;


    let id = selectedItems.keys().next().value;
    console.log(id);
    let index = 0;
    while(items[index].id !== id) {
      index++;
    }

    props.onChange(items[index]);

    selectedItems = new Set();

    setModalHeader("Изменение");
    setModalConfirm("Изменить");
    setEdit(true);
    setVisible(true);
  }

  function removeHandler() {
    let queries = [];
    for(let id of selectedItems.values()) {
      queries.push(DataService.remove(props.url, id));
    }

    Promise.all(queries).then(() => {
      loadItems()
    });
  }

  function confirmHandler() {
    saveData();
    setVisible(false);
  }

  function closeHandler() {
    setVisible(false);
  }

  return (
    <div className="m-3">
      <Toolbar addHandler={addHandler} changeHandler={changeHandler} removeHandler={removeHandler} />
      <Table headers={props.headers} items={items} selected={selectedItems} toggleSelect={toggleSelect} selectable={true}/>
      <Modal 
        visible={visible} 
        closeHandler={closeHandler}
        header={modalHeader}
        confirm={modalConfirm}
        confirmHandler={confirmHandler}
      >
        {props.children}
      </Modal>
    </div>
  )
}