package ru.ip.labs.labs.films.controller;

import org.springframework.web.bind.annotation.*;
import ru.ip.labs.labs.WebConfiguration;
import ru.ip.labs.labs.films.dto.ActorDTO;
import ru.ip.labs.labs.films.service.ActorService;

import javax.validation.Valid;
import java.util.List;

@RestController
@RequestMapping(WebConfiguration.REST_API + "/actors")
public class ActorController {
    private final ActorService actorService;

    public ActorController(ActorService actorService) {
        this.actorService = actorService;
    }

    @GetMapping("/{id}")
    public ActorDTO getActor(@PathVariable Long id) {
        return new ActorDTO(actorService.findActor(id));
    }

    @GetMapping("")
    public List<ActorDTO> getActors() {
        return actorService.findAllActors().stream().map(ActorDTO::new).toList();
    }

    @PostMapping("")
    public ActorDTO createActor(@RequestBody @Valid ActorDTO actor) {
        return new ActorDTO(actorService.addActor(actor.getName(), actor.getSurname()));
    }

    @PatchMapping("/{id}")
    public ActorDTO updateActor(@PathVariable Long id, @RequestBody @Valid ActorDTO actor) {
        return new ActorDTO(actorService.updateActor(id, actor.getName(), actor.getSurname()));
    }

    @DeleteMapping("/{id}")
    public ActorDTO deleteActor(@PathVariable Long id) {
        return new ActorDTO(actorService.deleteActor(id));
    }
}
