package ru.ip.labs.labs.films.controller;

import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.*;
import ru.ip.labs.labs.films.dto.ActorDTO;
import ru.ip.labs.labs.films.service.ActorService;

import javax.validation.Valid;

@Controller
@RequestMapping("/actor")
public class ActorMvcController {
    private ActorService actorService;
    public ActorMvcController(ActorService actorService) {
        this.actorService = actorService;
    }
    @GetMapping
    public String getActors(Model model) {
        model.addAttribute("actors",
                actorService.findAllActors().stream()
                        .map(ActorDTO::new).toList());

        model.addAttribute("actorDTO", new ActorDTO());

        return "actors-catalog";
    }

    @PostMapping(value = {"", "/{id}"})
    public String saveActor(@PathVariable(required = false) Long id,
                            @ModelAttribute @Valid ActorDTO actorDTO,
                            BindingResult bindingResult,
                            Model model) {
        if (bindingResult.hasErrors()) {
            model.addAttribute("errors", bindingResult.getAllErrors());
            return "actors-catalog";
        }
        if (id == null || id <= 0) {
            actorService.addActor(actorDTO.getName(), actorDTO.getSurname());
        } else {
            actorService.updateActor(id, actorDTO.getName(), actorDTO.getSurname());
        }
        return "redirect:/actor";
    }

    @PostMapping("/delete/{id}")
    public String deleteGenre(@PathVariable Long id) {
        actorService.deleteActor(id);
        return "redirect:/actor";
    }
}
