package ru.ip.labs.labs.films.controller;

import org.springframework.web.bind.annotation.*;
import ru.ip.labs.labs.WebConfiguration;
import ru.ip.labs.labs.films.dto.FilmDTO;
import ru.ip.labs.labs.films.models.Film;
import ru.ip.labs.labs.films.models.Genre;
import ru.ip.labs.labs.films.service.FilmsService;

import javax.validation.Valid;
import java.util.Iterator;
import java.util.List;

@RestController
@RequestMapping(WebConfiguration.REST_API + "/films")
public class FilmController {
    private final FilmsService filmService;

    public FilmController(FilmsService filmService) {
        this.filmService = filmService;
    }

    @GetMapping("/{id}")
    public FilmDTO getFilm(@PathVariable Long id) {
        return new FilmDTO(filmService.findFilm(id));
    }

    @GetMapping("")
    public List<FilmDTO> getFilms() {
        return filmService.findAllFilms().stream().map(FilmDTO::new).toList();
    }

    @PostMapping("")
    public FilmDTO createFilm(@RequestBody @Valid FilmDTO film) {
        return new FilmDTO(filmService.addFilm(film.getName(), film.getGenre(), film.getFullNames()));
    }

    @PatchMapping("/{id}")
    public FilmDTO updateFilm(@PathVariable Long id,
                              @RequestBody @Valid FilmDTO film) {
        return new FilmDTO(filmService.updateFilm(id, film.getName(), film.getGenre(), film.getFullNames()));
    }

    @PatchMapping("/add_genre/{id}")
    public FilmDTO addGenre(@PathVariable Long id, @RequestParam Long genre_id) {
        return new FilmDTO(filmService.addGenre(id, genre_id));
    }

    @PatchMapping("/add_actor/{id}")
    public FilmDTO addActor(@PathVariable Long id, @RequestParam Long actor_id) {
        return new FilmDTO(filmService.addActor(id, actor_id));
    }

    @DeleteMapping("/{id}")
    public FilmDTO deleteFilm(@PathVariable Long id) {
        return new FilmDTO(filmService.deleteFilm(id));
    }
}
