package ru.ip.labs.labs.films.controller;

import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.*;
import ru.ip.labs.labs.films.dto.FilmDTO;
import ru.ip.labs.labs.films.models.Film;
import ru.ip.labs.labs.films.service.ActorService;
import ru.ip.labs.labs.films.service.FilmsService;
import ru.ip.labs.labs.films.service.GenreService;

import javax.validation.Valid;

@Controller
@RequestMapping("/film")
public class FilmMvcController {
    private FilmsService filmService;
    private GenreService genreService;
    private ActorService actorService;
    public FilmMvcController(FilmsService filmService, GenreService genreService, ActorService actorService) {
        this.filmService = filmService;
        this.genreService = genreService;
        this.actorService = actorService;
    }
    @GetMapping
    public String getFilms(Model model) {
        model.addAttribute("films",
                filmService.findAllFilms().stream()
                        .map(FilmDTO::new).toList());

        model.addAttribute("filmDTO", new FilmDTO());
        model.addAttribute("allGenres",
                genreService.findAllGenres().stream()
                        .map(g -> g.getName()).toList());
        model.addAttribute("allActors",
                actorService.findAllActors().stream()
                        .map(a -> a.getName() + " " + a.getSurname()).toList());

        return "films-catalog";
    }

    @PostMapping(value = {"", "/{id}"})
    public String saveFilm(@PathVariable(required = false) Long id,
                            @ModelAttribute @Valid FilmDTO filmDTO,
                            BindingResult bindingResult,
                            Model model) {
        if (bindingResult.hasErrors()) {
            model.addAttribute("errors", bindingResult.getAllErrors());
            return "films-catalog";
        }

        if (id == null || id <= 0) {
            filmService.addFilm(filmDTO.getName(), filmDTO.getGenre(), filmDTO.getFullNames());
        } else {
            filmService.updateFilm(id, filmDTO.getName(), filmDTO.getGenre(), filmDTO.getFullNames());
        }
        return "redirect:/film";
    }

    @PostMapping("/delete/{id}")
    public String deleteGenre(@PathVariable Long id) {
        filmService.deleteFilm(id);
        return "redirect:/film";
    }
}
