package ru.ip.labs.labs.films.controller;

import org.springframework.web.bind.annotation.*;
import ru.ip.labs.labs.WebConfiguration;
import ru.ip.labs.labs.films.dto.FilmDTO;
import ru.ip.labs.labs.films.dto.GenreDTO;
import ru.ip.labs.labs.films.models.Film;
import ru.ip.labs.labs.films.models.Genre;
import ru.ip.labs.labs.films.service.FilmsService;
import ru.ip.labs.labs.films.service.GenreService;

import javax.validation.Valid;
import java.util.List;

@RestController
@RequestMapping(WebConfiguration.REST_API + "/genres")
public class GenreController {
    private final GenreService genreService;


    public GenreController(GenreService genreService) {
        this.genreService = genreService;
    }

    @GetMapping("/{id}")
    public GenreDTO getGenre(@PathVariable Long id) {
        return new GenreDTO(genreService.findGenre(id));
    }

    @GetMapping("")
    public List<GenreDTO> getGenres() {
        return genreService.findAllGenres().stream().map(GenreDTO::new).toList();
    }

    @PostMapping("")
    public GenreDTO createGenre(@RequestBody @Valid GenreDTO genre) {
        return new GenreDTO(genreService.addGenre(genre.getName()));
    }

    @PatchMapping("/{id}")
    public GenreDTO updateGenre(@PathVariable Long id,
                                @RequestBody @Valid GenreDTO genre) {
        return new GenreDTO(genreService.updateGenre(id, genre.getName()));
    }

    @DeleteMapping("/{id}")
    public GenreDTO deleteGenre(@PathVariable Long id) {
        return new GenreDTO(genreService.deleteGenre(id));
    }
}
