package ru.ip.labs.labs.films.controller;

import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.*;
import ru.ip.labs.labs.films.dto.GenreDTO;
import ru.ip.labs.labs.films.service.GenreService;

import javax.validation.Valid;

@Controller
@RequestMapping("/genre")
public class GenreMvcController {
    private GenreService genreService;
    public GenreMvcController(GenreService genreService) {
        this.genreService = genreService;
    }
    @GetMapping
    public String getGenres(Model model) {
        model.addAttribute("genres",
                genreService.findAllGenres().stream()
                        .map(GenreDTO::new).toList());

        model.addAttribute("genreDTO", new GenreDTO());

        return "genres-catalog";
    }

    @PostMapping(value = {"", "/{id}"})
    public String saveGenre(@PathVariable(required = false) Long id,
                              @ModelAttribute @Valid GenreDTO genreDTO,
                              BindingResult bindingResult,
                              Model model) {
        if (bindingResult.hasErrors()) {
            model.addAttribute("errors", bindingResult.getAllErrors());
            return "genres-catalog";
        }
        if (id == null || id <= 0) {
            genreService.addGenre(genreDTO.getName());
        } else {
            genreService.updateGenre(id, genreDTO.getName());
        }
        return "redirect:/genre";
    }

    @PostMapping("/delete/{id}")
    public String deleteGenre(@PathVariable Long id) {
        genreService.deleteGenre(id);
        return "redirect:/genre";
    }
}
