package ru.ip.labs.labs.films.dto;

import org.hibernate.annotations.LazyCollection;
import org.hibernate.annotations.LazyCollectionOption;
import ru.ip.labs.labs.films.models.Actor;
import ru.ip.labs.labs.films.models.Film;
import ru.ip.labs.labs.films.models.Genre;

import javax.persistence.FetchType;
import javax.persistence.JoinColumn;
import javax.persistence.JoinTable;
import javax.persistence.ManyToMany;
import java.util.List;

public class FilmDTO {
    private Long id;
    private String name;
    private List<String> genre;
    private List<String> fullNames;

    public FilmDTO() {}

    public FilmDTO(Film film) {
        this.id = film.getId();
        this.name = film.getName();
        if(film.getGenres() != null) {
            this.genre = film.getGenres().stream().map(Genre::getName).toList();
        }
        if(film.getActors() != null) {
            this.fullNames = film.getActors().stream().map(actor -> actor.getName() + " " + actor.getSurname()).toList();
        }
    }

    public FilmDTO(Long id, String name) {
        this.id = id;
        this.name = name;
    }

    public Long getId() {
        return id;
    }

    public String getName() {
        return name;
    }

    public void setName(String name) {
        this.name = name;
    }

    public List<String> getFullNames() {
        return fullNames;
    }

    public void setFullNames(List<String> fullNames) {
        this.fullNames = fullNames;
    }

    public List<String> getGenre() {
        return genre;
    }

    public void setGenre(List<String> genre) {
        this.genre = genre;
    }

    @Override
    public String toString() {
        String res = "\nFilm{" +
                "id: " + id + "," +
                "name: " + name + "," +
                "genres:" + (genre == null ? "[]" : genre.toString()) + "}";

        return res;
    }
}
