package ru.ip.labs.labs.films.models;
import org.hibernate.annotations.LazyCollection;
import org.hibernate.annotations.LazyCollectionOption;
import org.hibernate.annotations.OnDelete;
import org.hibernate.annotations.OnDeleteAction;

import javax.persistence.*;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

@Entity
public class Film {
    @Id
    @GeneratedValue(strategy = GenerationType.AUTO)
    private Long id;
    private String name;

    @ManyToMany
    @LazyCollection(LazyCollectionOption.FALSE)
    @JoinTable(name="films_genres",
            joinColumns = @JoinColumn(name="film_id"),
            inverseJoinColumns = @JoinColumn(name="genre_id")
    )
    private List<Genre> genres;

    @ManyToMany(fetch = FetchType.EAGER)
    @JoinTable(name="actors_films",
            joinColumns = @JoinColumn(name="film_id"),
            inverseJoinColumns = @JoinColumn(name="actor_id")
    )
    private List<Actor> actors;

    public Film() {}

    public Film(String name) {
        this.name = name;
    }

    public Long getId() {
        return id;
    }

    public String getName() {
        return name;
    }

    public void setName(String name) {
        this.name = name;
    }

    public void addGenre(Genre g) {
        if(genres == null) genres = new ArrayList<>();
        genres.add(g);
    }

    public void deleteGenres() {
        genres = null;
    }

    public void deleteActors() {
        actors = null;
    }

    public void addActor(Actor actor) {
        if(actors == null) actors = new ArrayList<>();
        actors.add(actor);
    }

    public List<Actor> getActors() {
        return actors;
    }

    public List<Genre> getGenres() {
        return genres;
    }

    @Override
    public String toString() {
        String res = "\nFilm{" +
                "id: " + id + "," +
                "name: " + name + "," +
                "genres:" + (genres == null ? "[]" : genres.toString()) + "}"
                ;

        /*
        Iterator<Genre> iter = genres.iterator();
        while(iter.hasNext()) {
            Genre curr = iter.next();
            res += "{ id: " + curr.getId() + ", name: " + curr.getName() + (iter.hasNext() ? "}, " : "}");
        }

        res += "]\n}";*/
        return res;
    }
}
