package ru.ip.labs.labs.films.service;

import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.StringUtils;
import ru.ip.labs.labs.films.models.Actor;
import ru.ip.labs.labs.films.models.Film;
import ru.ip.labs.labs.films.repository.ActorRepository;

import javax.persistence.EntityManager;
import javax.persistence.EntityNotFoundException;
import javax.persistence.PersistenceContext;
import java.io.IOException;
import java.util.List;
import java.util.Optional;

@Service
public class ActorService {

    private ActorRepository repo;

    public ActorService(ActorRepository repo) {
        this.repo = repo;
    }

    @Transactional
    public Actor addActor(String name, String surname) {
        if (!StringUtils.hasText(name)) {
            throw new IllegalArgumentException("Student name is null or empty");
        }

        Actor actor = new Actor(name, surname);

        return repo.save(actor);
    }
    @Transactional
    public Actor addActor(String name, String surname, String path) {
        if (!StringUtils.hasText(name)) {
            throw new IllegalArgumentException("Student name is null or empty");
        }

        Actor actor = new Actor(name, surname);
        try {
            actor.setPhoto(path);
        } catch(IOException err) {
//            System.out.println(err.getMessage());
            throw new RuntimeException(err);
        }
        return repo.save(actor);
    }

    @Transactional(readOnly = true)
    public Actor findActor(Long id) {
        final Optional<Actor> actor = repo.findById(id);
        if (actor.isEmpty()) {
            throw new EntityNotFoundException(String.format("Actor with id [%s] is not found", id));
        }
        return actor.get();
    }

    @Transactional(readOnly = true)
    public Actor findByFullName(String name, String surname) {
        if(!StringUtils.hasText(name)) {
            throw new IllegalArgumentException("Actor name is null or empty");
        }

        if(!StringUtils.hasText(surname)) {
            throw new IllegalArgumentException("Actor surname is null or empty");
        }

        return repo.findActorByNameAndSurname(name, surname);
    }

    @Transactional(readOnly = true)
    public List<Actor> findAllActors() {
        return repo.findAll();
    }

    @Transactional
    public Actor updateActor(Long id, String name, String surname) {
        if (!StringUtils.hasText(name)) {
            throw new IllegalArgumentException("Actor name is null or empty");
        }
        final Optional<Actor> currentActorOpt = repo.findById(id);

        if(currentActorOpt.isEmpty()) {
            return null;
        }

        final Actor currentActor = currentActorOpt.get();

        if(name != null) currentActor.setName(name);
        if(surname != null) currentActor.setSurname(surname);

        return repo.save(currentActor);
    }

    @Transactional
    public Actor deleteActor(Long id) {
        final Optional<Actor> currentActor = repo.findById(id);
        if(currentActor.isEmpty()) {
            return null;
        }

        repo.deleteById(id);
        return currentActor.get();
    }

    @Transactional
    public void deleteAllActors() {
        repo.deleteAll();
    }
}
