package ru.ip.labs.labs.films.service;

import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.StringUtils;
import ru.ip.labs.labs.films.models.Genre;
import ru.ip.labs.labs.films.repository.GenreRepository;

import javax.persistence.EntityNotFoundException;
import java.util.List;
import java.util.Optional;

@Service
public class GenreService {

    private GenreRepository repo;

    public GenreService(GenreRepository repo) {
        this.repo = repo;
    }
    @Transactional
    public Genre addGenre(String name) {
        if (!StringUtils.hasText(name)) {
            throw new IllegalArgumentException("Genre name is null or empty");
        }

        Genre genre = new Genre(name);
        return repo.save(genre);
    }

    @Transactional(readOnly = true)
    public Genre findGenre(Long id) {
        final Optional<Genre> genre = repo.findById(id);
        if (genre.isEmpty()) {
            throw new EntityNotFoundException(String.format("Genre with id [%s] is not found", id));
        }
        return genre.get();
    }

    @Transactional(readOnly = true)
    public List<Genre> findAllGenres() {
        return repo.findAll();
    }

    @Transactional
    public Genre updateGenre(Long id, String name) {
        if (!StringUtils.hasText(name)) {
            throw new IllegalArgumentException("Film name is null or empty");
        }
        final Optional<Genre> currentGenreOpt = repo.findById(id);

        if(currentGenreOpt.isEmpty()) {
            return null;
        }

        final Genre currentGenre = currentGenreOpt.get();

        currentGenre.setName(name);
        return repo.save(currentGenre);
    }

    @Transactional
    public Genre findByName(String name) {
        if (!StringUtils.hasText(name)) {
            throw new IllegalArgumentException("Genre name is null or empty");
        }
        return repo.findByName(name);
    }

    @Transactional
    public Genre deleteGenre(Long id) {
       Optional<Genre> genre = repo.findById(id);
       if(genre.isEmpty()) {
           return null;
       }

       repo.deleteById(id);
       return genre.get();
    }

    @Transactional
    public void deleteAllGenres() {
        repo.deleteAll();
    }
}