package ru.ip.labs.labs;

import org.junit.jupiter.api.Assertions;
import org.junit.jupiter.api.Test;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;
import ru.ip.labs.labs.films.models.Actor;
import ru.ip.labs.labs.films.models.Film;
import ru.ip.labs.labs.films.models.Genre;
import ru.ip.labs.labs.films.service.ActorService;
import ru.ip.labs.labs.films.service.FilmsService;
import ru.ip.labs.labs.films.service.GenreService;

import javax.persistence.EntityNotFoundException;
import java.util.List;

@SpringBootTest
public class JpaActorsTest {
    private static final Logger log = LoggerFactory.getLogger(JpaFilmsTest.class);
    @Autowired
    FilmsService filmService;

    @Autowired
    ActorService actorService;

    @Test
    void testActorCreate() {
        filmService.deleteAllFilms();
        actorService.deleteAllActors();
        final Actor actor = actorService.addActor("Victor", "Ivanov");
        log.info(actor.toString());
        Assertions.assertNotNull(actor.getId());
    }

    @Test
    void testActorReadNotFound() {
        actorService.deleteAllActors();
        Assertions.assertThrows(EntityNotFoundException.class, () -> actorService.findActor(-1L));
    }

    @Test
    void findActor() {
        filmService.deleteAllFilms();
        actorService.deleteAllActors();
        final Actor actor = actorService.addActor("Victor", "Ivanov");
        log.info(actor.toString());
        final Actor findActor = actorService.findActor(actor.getId());
        log.info(findActor.toString());
        Assertions.assertEquals(actor.toString(), findActor.toString());
    }

    @Test
    void testReadAll() {
        filmService.deleteAllFilms();
        actorService.deleteAllActors();
        final Actor actor = actorService.addActor("Victor", "Ivanov");
        log.info(actor.toString());
        final List<Actor> actors = actorService.findAllActors();
        Assertions.assertEquals(actors.size(), 1);
    }

    @Test
    void updateActor() {
        filmService.deleteAllFilms();
        actorService.deleteAllActors();
        final Actor actor = actorService.addActor("Victor", "Ivanov");
        actorService.updateActor(actor.getId(), "Terminator", null);
        final Actor findActor = actorService.findActor(actor.getId());
        log.info(findActor.toString());
        Assertions.assertEquals(findActor.getName() + " " + findActor.getSurname(), "Terminator Ivanov");
    }

    @Test
    void deleteActor() {
        filmService.deleteAllFilms();
        actorService.deleteAllActors();
        final Actor actor = actorService.addActor("Victor", "Ivanov");
        log.info(actor.toString());
        actorService.deleteActor(actor.getId());
        Assertions.assertThrows(EntityNotFoundException.class, () -> actorService.findActor(actor.getId()));
    }

    @Test
    void addPhoto() {
        filmService.deleteAllFilms();
        actorService.deleteAllActors();
        final Actor actor = actorService.addActor("Victor", "Ivanov", "C:\\Users\\parap\\OneDrive\\Рабочий стол\\work\\ferrari.jpg");
        Actor findActor = actorService.findActor(actor.getId());
        Assertions.assertNotEquals(findActor.getPhoto().length, 0);
    }

    @Test
    void addFilm() {
        filmService.findAllFilms();
        actorService.deleteAllActors();
        final Actor actor = actorService.addActor("Victor", "Ivanov");
        final Film film = filmService.addFilm("Terminator 2");

        filmService.addActor(film.getId(), actor.getId());

        final Actor findActor = actorService.findActor(actor.getId());
        Assertions.assertEquals(findActor.getFilms().get(0).getId(), film.getId());
    }
}
