package ru.ip.labs.labs;
import org.junit.jupiter.api.Assertions;
import org.junit.jupiter.api.Test;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;
import ru.ip.labs.labs.films.models.Actor;
import ru.ip.labs.labs.films.models.Film;
import ru.ip.labs.labs.films.models.Genre;
import ru.ip.labs.labs.films.service.ActorService;
import ru.ip.labs.labs.films.service.FilmsService;
import ru.ip.labs.labs.films.service.GenreService;

import javax.persistence.EntityNotFoundException;
import java.util.List;

@SpringBootTest
public class JpaFilmsTest {
    private static final Logger log = LoggerFactory.getLogger(JpaFilmsTest.class);
    @Autowired
    FilmsService filmService;

    @Autowired
    GenreService genreService;

    @Autowired
    ActorService actorService;

    @Test
    void testFilmCreate() {
        filmService.deleteAllFilms();
        final Film film = filmService.addFilm("Terminator 2");
        log.info(film.toString());
        Assertions.assertNotNull(film.getId());
    }

    @Test
    void testFilmReadNotFound() {
        filmService.deleteAllFilms();
        Assertions.assertThrows(EntityNotFoundException.class, () -> filmService.findFilm(-1L));
    }

    @Test
    void findFilm() {
        filmService.deleteAllFilms();
        final Film film = filmService.addFilm("Terminator 2");
        log.info(film.toString());
        final Film findFilm = filmService.findFilm(film.getId());
        log.info(findFilm.toString());
        Assertions.assertEquals(film.toString(), findFilm.toString());
    }

    @Test
    void updateFilm() {
        filmService.deleteAllFilms();
        final Film film = filmService.addFilm("Terminator 2");
        filmService.updateFilm(film.getId(), "Terminator 3");
        final Film findFilm = filmService.findFilm(film.getId());
        log.info(findFilm.toString());
        Assertions.assertEquals(findFilm.getName(), "Terminator 3");
    }

    @Test
    void testReadAll() {
        filmService.deleteAllFilms();
        final Film film = filmService.addFilm("Terminator 2");
        log.info(film.toString());
        final List<Film> films = filmService.findAllFilms();
        Assertions.assertEquals(films.size(), 1);
    }

    @Test
    void deleteFilm() {
        filmService.deleteAllFilms();
        final Film film = filmService.addFilm("Terminator 2");
        log.info(film.toString());
        filmService.deleteFilm(film.getId());
        Assertions.assertThrows(EntityNotFoundException.class, () -> filmService.findFilm(film.getId()));
    }

    @Test
    void addActor() {
        filmService.deleteAllFilms();
        genreService.deleteAllGenres();
        final Film film = filmService.addFilm("Terminator 2");
        final Actor actor = actorService.addActor("Victor", "Ivanov");

        filmService.addActor(film.getId(), actor.getId());

        final Film findFilm = filmService.findFilm(film.getId());
        Assertions.assertEquals(findFilm.getActors().get(0).getId(), actor.getId());
    }
}
