package ru.ip.labs.labs;

import org.junit.jupiter.api.Assertions;
import org.junit.jupiter.api.Test;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;
import ru.ip.labs.labs.films.models.Genre;
import ru.ip.labs.labs.films.service.FilmsService;
import ru.ip.labs.labs.films.service.GenreService;

import javax.persistence.EntityNotFoundException;
import java.util.List;

@SpringBootTest
public class JpaGenresTest {
    private static final Logger log = LoggerFactory.getLogger(JpaFilmsTest.class);
    @Autowired
    FilmsService filmService;

    @Autowired
    GenreService genreService;

    @Test
    void testGenreCreate() {
        genreService.deleteAllGenres();
        final Genre genre = genreService.addGenre("Comedy");
        log.info(genre.toString());
        Assertions.assertNotNull(genre.getId());
    }

    @Test
    void testGenreReadNotFound() {
        genreService.deleteAllGenres();
        Assertions.assertThrows(EntityNotFoundException.class, () -> genreService.findGenre(-1L));
    }

    @Test
    void findGenre() {
        genreService.deleteAllGenres();
        final Genre genre = genreService.addGenre("Comedy");
        log.info(genre.toString());
        final Genre findGenre = genreService.findGenre(genre.getId());
        log.info(findGenre.toString());
        Assertions.assertEquals(genre.toString(), findGenre.toString());
    }

    @Test
    void testReadAll() {
        genreService.deleteAllGenres();
        final Genre genre = genreService.addGenre("Comedy");
        log.info(genre.toString());
        final List<Genre> genres = genreService.findAllGenres();
        Assertions.assertEquals(genres.size(), 1);
    }

    @Test
    void updateGenre() {
        genreService.deleteAllGenres();
        final Genre genre = genreService.addGenre("Comedy");
        genreService.updateGenre(genre.getId(), "Drama");
        final Genre findGenre = genreService.findGenre(genre.getId());
        log.info(findGenre.toString());
        Assertions.assertEquals(findGenre.getName(), "Drama");
    }

    @Test
    void deleteGenre() {
        genreService.deleteAllGenres();
        final Genre genre = genreService.addGenre("Comedy");
        log.info(genre.toString());
        genreService.deleteGenre(genre.getId());
        Assertions.assertThrows(EntityNotFoundException.class, () -> filmService.findFilm(genre.getId()));
    }
}
